"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const epoxy_1 = require("../../epoxy");
const isGlob = require("is-glob");
const isValidPath = require("is-valid-path");
const graphql_1 = require("graphql");
const glob = require("glob");
const fs_1 = require("fs");
const extract_document_string_from_code_file_1 = require("../../utils/extract-document-string-from-code-file");
const graphql_import_1 = require("graphql-import");
console.log('importSchema:', graphql_import_1.default);
const GQL_EXTENSIONS = ['.graphql', '.graphqls', '.gql'];
const INVALID_SCHEMA_KINDS = [graphql_1.Kind.OPERATION_DEFINITION, graphql_1.Kind.FRAGMENT_DEFINITION];
function isGraphQLFile(globPath) {
    return GQL_EXTENSIONS.some(ext => globPath.endsWith(ext));
}
exports.isGraphQLFile = isGraphQLFile;
async function loadSchemaFile(filepath, options) {
    const content = fs_1.readFileSync(filepath, 'utf-8');
    console.log(`${filepath}: A`);
    if (content && content.trim() !== '') {
        console.log(`${filepath}: B`);
        if (/^\#.*import /i.test(content.trimLeft())) {
            const { importSchema } = await Promise.resolve().then(() => require('graphql-import'));
            console.log(`${filepath}: C`);
            console.log('importSchema!!!:', importSchema);
            //console.log(importSchema(filepath));
            return importSchema(filepath);
        }
        console.log(`${filepath}: D`);
        const foundDoc = await extract_document_string_from_code_file_1.extractDocumentStringFromCodeFile(new graphql_1.Source(content, filepath), options);
        console.log(`${filepath}: E`);
        if (foundDoc) {
            console.log(`${filepath}: F`);
            return foundDoc;
        }
        console.log(`${filepath}: G`);
        if (isGraphQLFile(filepath)) {
            console.log(`${filepath}: H`);
            return content;
        }
    }
    else {
        console.log(`${filepath}: I`);
        console['warn'](`Empty schema file found: "${filepath}", skipping...`);
    }
    console.log(`${filepath}: E`);
    return null;
}
class SchemaFromTypedefs {
    canHandle(globOrValidPath) {
        return isGlob(globOrValidPath) || isValidPath(globOrValidPath);
    }
    async handle(globPath, options) {
        let globFiles = glob.sync(globPath, { cwd: process.cwd() });
        // globFiles = [globFiles[11], globFiles[12]];
        if (!globFiles || globFiles.length === 0) {
            throw new Error(`Unable to find matching files for glob: ${globPath} in directory: ${process.cwd()}`);
        }
        console.log('globFiles:', globFiles);
        const filesContent$ = Promise.all(globFiles
            .map(async (filePath) => ({ filePath, content: await loadSchemaFile(filePath, options) })));
        console.log('filesContent length:', (await filesContent$).length);
        // console.log('filesContent 12th element:', (await filesContent$)[12]);
        const filesContent = (await filesContent$)
            .filter(file => {
            console.log('ZG');
            console.log(file);
            if (!file.content) {
                return false;
            }
            let node;
            try {
                node = graphql_1.parse(file.content);
            }
            catch (err) {
                console.error(`parse error in ${JSON.stringify(file)}:`, err);
                console.log(`file.content:`, file.content);
                console.log(`file:`, file);
                throw err;
            }
            const invalidSchemaDefinitions = node.definitions.filter(def => INVALID_SCHEMA_KINDS.includes(def.kind));
            if (invalidSchemaDefinitions.length === 0) {
                return true;
            }
            else {
                console['warn'](`File "${file.filePath}" was filtered because it contains an invalid GraphQL schema definition!`);
                return false;
            }
        });
        if (filesContent.length === 0) {
            throw new Error(`All found files for glob expression "${globPath}" are not valid or empty, please check it and try again!`);
        }
        return epoxy_1.mergeTypeDefs(filesContent.map(f => f.content));
    }
}
exports.SchemaFromTypedefs = SchemaFromTypedefs;
//# sourceMappingURL=schema-from-typedefs.js.map